@@ -15,12 +15,12 @@ use Petal::Functions;
 use Petal::Entities;
 use File::Spec;
 use Carp;
-use Safe;
 use Data::Dumper;
 use Scalar::Util;
 use strict;
 use warnings;
 use MKDoc::XML::Decode;
+use Petal::I18N;
 
 
 BEGIN
@@ -38,6 +38,10 @@ BEGIN
 use vars qw /@tokens @nodeStack/;
 
 
+# HTML errors?
+our $HTML_ERRORS = undef;
+
+
 # Encode / Decode info...
 our $DECODE_CHARSET = 'utf8';
 our $ENCODE_CHARSET = 'utf8'; # deprecated
@@ -90,13 +94,17 @@ our $DISK_CACHE = 1;
 our $MEMORY_CACHE = 1;
 
 
+# cache only mode
+our $CACHE_ONLY = 0;
+
+
 # prevents infinites includes...
 our $MAX_INCLUDES = 30;
 our $CURRENT_INCLUDES = 0;
 
 
 # this is for CPAN
-our $VERSION = '2.05';
+our $VERSION = '2.23';
 
 
 # The CodeGenerator class backend to use.
@@ -120,6 +128,9 @@ our $MT_NS       = 'metal';
 our $MT_NS_URI   = 'http://xml.zope.org/namespaces/metal';
 our $MT_NAME_CUR = 'main';
 
+# translation service, optional
+our $TranslationService = undef;
+
 
 # Displays the canonical template for template.xml.
 # You can set $INPUT using by setting the PETAL_INPUT environment variable.
@@ -129,6 +140,7 @@ sub main::canonical
     my $file = shift (@ARGV);
     local $Petal::DISK_CACHE = 0;
     local $Petal::MEMORY_CACHE = 0;
+    local $Petal::CACHE_ONLY = 0;
     local $Petal::INPUT  = $ENV{PETAL_INPUT}  || 'XML';
     local $Petal::OUTPUT = $ENV{PETAL_OUTPUT} || 'XHTML';
     print ${Petal->new ($file)->_canonicalize()};
@@ -143,6 +155,7 @@ sub main::code
     my $file = shift (@ARGV);
     local $Petal::DISK_CACHE = 0;
     local $Petal::MEMORY_CACHE = 0;
+    local $Petal::CACHE_ONLY = 0;
     print Petal->new ($file)->_code_disk_cached;
 }
 
@@ -155,6 +168,7 @@ sub main::lcode
     my $file = shift (@ARGV);
     local $Petal::DISK_CACHE = 0;
     local $Petal::MEMORY_CACHE = 0;
+    local $Petal::CACHE_ONLY = 0;
     print Petal->new ($file)->_code_with_line_numbers;
 }
 
@@ -233,6 +247,7 @@ sub taint              { exists $_[0]->{taint}              ? $_[0]->{taint}
 sub error_on_undef_var { exists $_[0]->{error_on_undef_var} ? $_[0]->{error_on_undef_var} : $ERROR_ON_UNDEF_VAR }
 sub disk_cache         { exists $_[0]->{disk_cache}         ? $_[0]->{disk_cache}         : $DISK_CACHE         }
 sub memory_cache       { exists $_[0]->{memory_cache}       ? $_[0]->{memory_cache}       : $MEMORY_CACHE       }
+sub cache_only         { exists $_[0]->{cache_only}         ? $_[0]->{cache_only}         : $CACHE_ONLY         }
 sub max_includes       { exists $_[0]->{max_includes}       ? $_[0]->{max_includes}       : $MAX_INCLUDES       }
 
 
@@ -329,20 +344,22 @@ sub process
     
     # ok, from there on we need to override any global variable with stuff
     # that might have been specified when constructing the object
-    local $TAINT              = defined $self->{taint}              ? $self->{taint}              : $TAINT;
-    local $ERROR_ON_UNDEF_VAR = defined $self->{error_on_undef_var} ? $self->{error_on_undef_var} : $ERROR_ON_UNDEF_VAR;
-    local $DISK_CACHE         = defined $self->{disk_cache}         ? $self->{disk_cache}         : $DISK_CACHE;
-    local $MEMORY_CACHE       = defined $self->{memory_cache}       ? $self->{memory_cache}       : $MEMORY_CACHE;
-    local $MAX_INCLUDES       = defined $self->{max_includes}       ? $self->{max_includes}       : $MAX_INCLUDES;
-    local $INPUT              = defined $self->{input}              ? $self->{input}              : $INPUT;
-    local $OUTPUT             = defined $self->{output}             ? $self->{output}             : $OUTPUT;
+    local $TAINT              = defined $self->{taint}               ? $self->{taint}               : $TAINT;
+    local $ERROR_ON_UNDEF_VAR = defined $self->{error_on_undef_var}  ? $self->{error_on_undef_var}  : $ERROR_ON_UNDEF_VAR;
+    local $DISK_CACHE         = defined $self->{disk_cache}          ? $self->{disk_cache}          : $DISK_CACHE;
+    local $MEMORY_CACHE       = defined $self->{memory_cache}        ? $self->{memory_cache}        : $MEMORY_CACHE;
+    local $CACHE_ONLY         = defined $self->{cache_only}          ? $self->{cache_only}          : $CACHE_ONLY;
+    local $MAX_INCLUDES       = defined $self->{max_includes}        ? $self->{max_includes}        : $MAX_INCLUDES;
+    local $INPUT              = defined $self->{input}               ? $self->{input}               : $INPUT;
+    local $OUTPUT             = defined $self->{output}              ? $self->{output}              : $OUTPUT;
     local $BASE_DIR           = defined $self->{base_dir} ? do { ref $self->{base_dir} ? undef : $self->{base_dir} } : $BASE_DIR;
     local @BASE_DIR           = defined $self->{base_dir} ? do { ref $self->{base_dir} ? @{$self->{base_dir}} : () } : @BASE_DIR;
-    local $LANGUAGE           = defined $self->{default_language}   ? $self->{default_language}   : $LANGUAGE;
-    local $DEBUG_DUMP         = defined $self->{debug_dump}         ? $self->{debug_dump}         : $DEBUG_DUMP;
-    local $DECODE_CHARSET     = defined $self->{decode_charset}     ? $self->{decode_charset}     : $DECODE_CHARSET;
+    local $LANGUAGE           = defined $self->{default_language}    ? $self->{default_language}    : $LANGUAGE;
+    local $DEBUG_DUMP         = defined $self->{debug_dump}          ? $self->{debug_dump}          : $DEBUG_DUMP;
+    local $DECODE_CHARSET     = defined $self->{decode_charset}      ? $self->{decode_charset}      : $DECODE_CHARSET;
+    local $TranslationService = defined $self->{translation_service} ? $self->{translation_service} : $TranslationService;
     # local $ENCODE_CHARSET     = defined $self->{encode_charset}     ? $self->{encode_charset}     : $ENCODE_CHARSET;
-    
+
     # prevent infinite includes from happening...
     my $current_includes = $CURRENT_INCLUDES;
     return "ERROR: MAX_INCLUDES : $CURRENT_INCLUDES" if ($CURRENT_INCLUDES > $MAX_INCLUDES);
@@ -361,7 +378,11 @@ sub process
 	$res = $coderef->($hash);
     };
     
-    $self->_handle_error ($@) if (defined $@ and $@);
+    if ( $CACHE_ONLY == 1 ){ return 1; }
+   
+    if (defined $@ and $@) { $res = $self->_handle_error ($@) }
+    elsif (defined $TranslationService && $CURRENT_INCLUDES == 1) { $res = Petal::I18N->process ($res) } 
+
     return $res;
 }
 
@@ -407,6 +428,29 @@ sub _handle_error
 {
     my $self = shift;
     my $error = shift;
+
+    $Petal::HTML_ERRORS and do {
+        my $res  = '<pre>';
+        $res    .= "Error: $error\n";
+        $res    .= "=============\n";
+        $res    .= "\n\n";
+        $res    .= "Petal object dump:\n";
+        $res    .= "==================\n";
+	$res    .= Dumper ($self);
+        $res    .= "\n\n";
+	$res    .= "Stack trace:\n";
+	$res    .= "============\n";
+	$res    .= Carp::longmess();
+	$res    .= "\n\n";
+        $res    .= "Template perl code dump:\n";
+        $res    .= "========================\n";
+
+        my $dump = eval { $self->_code_with_line_numbers() };
+        $res    .= ($dump) ? $dump : "(no dump available)";
+        
+        $res .= '</pre>';
+        return $res;
+    };
     
     $Petal::DEBUG_DUMP and do {
 	my $tmpdir  = File::Spec->tmpdir();
@@ -436,7 +480,7 @@ sub _handle_error
 	my $dump = eval { $self->_code_with_line_numbers() };
 	($dump) ? print ERROR $dump : print ERROR "(no dump available)";
 	
-	die "[PETAL ERROR] $error. Debug info written in $debug.";
+	die "[PETAL ERROR] $error. Debug info written in $debug";
     };
     
     ! $Petal::DEBUG_DUMP and do {
@@ -544,7 +588,7 @@ sub _file_data_ref
     if ($] > 5.007)
     {
 	my $encoding = Encode::resolve_alias ($DECODE_CHARSET) || 'utf8';
-	open FP, "<:$encoding", "$file_path" || die 'Cannot read-open $file_path';
+	open FP, "<:encoding($encoding)", "$file_path" or die "Cannot read-open $file_path";
     }
     else
     {
@@ -572,7 +616,7 @@ sub _file_data_ref
 sub _code_disk_cached
 {
     my $self = shift;
-    my $code = (defined $DISK_CACHE and $DISK_CACHE) ? Petal::Cache::Disk->get ($self->_file_path_with_macro) : undef;
+    my $code = (defined $DISK_CACHE and $DISK_CACHE) ? Petal::Cache::Disk->get ($self->_file_path_with_macro, $self->language) : undef;
     unless (defined $code)
     {
 	my $macro = $self->_macro() || $MT_NAME_CUR;
@@ -583,7 +627,7 @@ sub _code_disk_cached
 	my $data_ref = $self->_canonicalize;
 	load_code_generator();
 	$code = $CodeGenerator->process ($data_ref, $self);
-	Petal::Cache::Disk->set ($self->_file_path_with_macro, $code) if (defined $DISK_CACHE and $DISK_CACHE);
+	Petal::Cache::Disk->set ($self->_file_path_with_macro, $code, $self->language) if (defined $DISK_CACHE and $DISK_CACHE);
     }
     
     return $code;
@@ -596,33 +640,17 @@ sub _code_disk_cached
 sub _code_memory_cached
 {
     my $self = shift;
-    my $code = (defined $MEMORY_CACHE and $MEMORY_CACHE) ? Petal::Cache::Memory->get ($self->_file_path_with_macro) : undef;
+    my $code = (defined $MEMORY_CACHE and $MEMORY_CACHE) ? Petal::Cache::Memory->get ($self->_file_path_with_macro, $self->language) : undef;
     unless (defined $code)
     {
 	my $code_perl = $self->_code_disk_cached;
 	my $VAR1 = undef;
 	
-	if ($TAINT)
-	{
-	    # important line, don't remove
-	    ($code_perl) = $code_perl =~ m/^(.+)$/s;
-	    die "\$code_perl is empty after untainting!" unless defined $code_perl && $code_perl;
-	    my $cpt = Safe->new ("Petal::CPT");
-	    $cpt->permit ('entereval');
-	    $cpt->permit ('leaveeval');
-	    $cpt->permit ('require');
-	    $code = $cpt->reval($code_perl);
-	    confess ("Error in reval:\n" . $@ . "\n" . $self->_code_with_line_numbers) if $@;
-	    warn "\$code is empty after reval.\n" . Dumper($code, $Petal::CPT::VAR1, length($code_perl)) unless $code;
-	}
-	else
-	{
-	    eval "$code_perl";
-	    confess ($@ . "\n" . $self->_code_with_line_numbers) if $@;
-	    $code = $VAR1;
-	}
+	eval "$code_perl";
+	confess ($@ . "\n" . $self->_code_with_line_numbers) if $@;
+	$code = $VAR1;
 	
-	Petal::Cache::Memory->set ($self->_file_path_with_macro, $code) if (defined $MEMORY_CACHE and $MEMORY_CACHE);	
+	Petal::Cache::Memory->set ($self->_file_path_with_macro, $code, $self->language) if (defined $MEMORY_CACHE and $MEMORY_CACHE);	
     }
     
     return $code;
@@ -664,19 +692,6 @@ sub _canonicalize
 Petal - Perl Template Attribute Language - TAL for Perl!
 
 
-=head1 IMPORTANT NOTE 
-
-From version 2.00 onwards Petal *requires* that you use well-formed XML. This
-is because Petal now uses L<MKDoc::XML::TreeBuilder> rather than
-L<HTML::TreeBuilder> and L<XML::Parser>.
-
-In particular, this version of Petal *CAN* break backwards compatibility if you
-were using Petal's HTML mode will non well formed XHTML.
-
-If you still want to use broken XHTML, you can Petal 2.00 in conjunction with
-L<Petal::Parser::HTB> which has been created for this purpose.
-
-
 =head1 SYNOPSIS
 
 in your Perl code:
@@ -743,52 +758,15 @@ Let's say you have the following Perl code:
     local $Petal::OUTPUT = 'XHTML';
 
     my $template = new Petal ('foo.xhtml');
-    template->process ( my_var => my_var() );
+    $template->process ( my_var => some_object() );
 
 some_object() is a subroutine that returns some kind of object, may it be a scalar,
 object, array referebce or hash reference. Let's see what we can do...
 
 
-=head2 Version 1: Prototype
+=head2 Version 1: WYSIWYG friendly prototype.
 
-    <!--? This is a template comment.
-          It will not appear in the output -->
-    <html xmlns:tal="http://purl.org/petal/1.0/">
-      <body>
-        This is the variable 'my_var' : ${my_var}.
-      </body>
-    </html>
-
-
-And if C<my_var> contained I<Hello World>, Petal would have outputted:
-
-    <html>
-      <body>
-        This is the variable 'my_var' : Hello World.
-      </body>
-    </html>
-
-
-Now let's say that C<my_var> is a hash reference as follows:
-
-    $VAR1 = { hello_world => 'Hello, World' }
-
-
-To output the same result, you would write:
-
-    This is the variable 'my_var' : ${my_var/hello_world}.
-
-
-=head2 Version 2: WYSIWYG friendly.
-
-The problem with the above page is that when you edit it with a WYSIWYG editor,
-or simply open it in your browser, you will see:
-
-    This is the variable 'my_var' : ${my_var/hello_world}.
-
-Ideally you don't want your graphic designers to worry about
-variable names... and that's where TAL kicks in. Using TAL
-you can do:
+Using TAL you can do:
 
     This is the variable 'my_var' :
     <span tal:replace="my_var/hello_world">Hola, Mundo!</span>
@@ -798,29 +776,26 @@ frontpage, dreamweaver, adobe golive...) and work with less risk of damaging
 your petal commands.
 
 
-=head2 Version 3: Object-oriented version
+=head2 Version 2: Object-oriented version
 
 Let's now say that C<my_var> is actually an object with a method hello_world()
-that returns I<Hello World>. To output the same result, your line:
+that returns I<Hello World>. To output the same result, your line, which was:
 
     <span tal:replace="my_var/hello_world">Hola, Mundo!</span>
 
-Would become:
-
-    <span tal:replace="my_var/hello_world">Hola, Mundo!</span>
-
-Look carefully at those two lines. That's right. There are identical. Petal
-lets you access hashes and objects in an entirely transparent way.
+Would need to be... EXACTLY the same. Petal lets you access hashes and objects
+in an entirely transparent way and tries to automagically do The Right Thing
+for you.
 
 This high level of polymorphism means that in most cases you can maintain your
 code, swap hashes for objects, and not change a single line of your template
 code.
 
 
-=head2 Version 4: Personalizable
+=head2 Version 3: Personalizable
 
-Now let's say that your method some_object() can take an optional
-argument so that C<$my_var->hello_world ('Jack')> returns I<Hello Jack>.
+Now let's say that your method hello_world() can take an optional
+argument so that C<$some_object-E<gt>hello_world ('Jack')> returns I<Hello Jack>.
 
 You would write:
 
@@ -849,7 +824,7 @@ You cannot write nested expressions such as:
 This will NOT work. At least, not yet.
 
 
-=head2 Version 5: Internationalized
+=head2 Version 4: Internationalized
 
 Let's say that you have a directory called C<hello_world> with the following
 files:
@@ -864,11 +839,18 @@ You can use Petal as follows in your Perl code:
     local $Petal::OUTPUT = 'XHTML';
 
     my $template = new Petal ( file => 'hello_world', lang => 'fr-CA' );
-    print $template->process ( my_var => my_var() );
+    print $template->process ( my_var => some_object() );
 
-What will happen is that the C<$template> object will try to find a file named
-C<fr-CA>, then C<fr>, then will default to <en>. It should work fine for
-includes, too!
+What will happen is that the C<$template> object will look in the
+C<hello_world> directory and try to find a file named C<fr-CA.xhtml>, then
+C<fr.xhtml>, then will default to C<en.xhtml>. It works fine for includes, too!
+
+These internationalized templates can have whatever file-extension you like,
+Petal searches on the first part of the filename.  So you can call them
+C<fr.html>, C<fr.xml>, C<fr.xhtml> or use whatever convention suits you.
+
+NOTE: There is now support for ZPT-like i18n attributes, which should
+provide a much nicer framework. See L<Petal::I18N> for details.
 
 
 TIP:
@@ -925,9 +907,9 @@ directory list use an arrayref.
 
 Defines the format of the template files.  Recognised values are:
 
-  'HTML'  - Petal will use HTML::TreeBuilder to parse the template
-  'XHTML' - Alias for 'HTML'
-  'XML'   - Petal will use XML::Parser to parse the template
+  'HTML'  - Alias for 'XHTML'
+  'XHTML' - Petal will use Petal::Parser to parse the template
+  'XML'   - Petal will use Petal::Parser to parse the template
 
 
 =head2 output => 'HTML' | 'XHTML' | 'XML' (default: 'XML')
@@ -976,6 +958,13 @@ If set to C<false>, Petal will not use the C<Petal::Cache::Disk> module.
 If set to C<false>, Petal will not use the C<Petal::Cache::Memory> module.
 
 
+=head2 cache_only => I<true> | I<false> (default: I<false>)
+
+If set to C<true>, Petal will return true after having compiled a template into
+perl code and a subroutine , and optionally using disk_cache or memory_cache if
+either is set.
+
+
 =head2 max_includes => I<number> (default: 30)
 
 The maximum number of recursive includes before Petal stops processing.  This
@@ -986,7 +975,8 @@ is to guard against accidental infinite recursions.
 
 If this option is true, when Petal cannot process a template it will
 output lots of debugging information in a temporary file which you can
-inspect.
+inspect.  The location for this file is wherever File::Spec->tmpdir()
+specifies as a temp directory (usually /tmp on a unix system).
 
 
 =head2 encode_charset => I<charset> (default: undef)
@@ -1011,30 +1001,10 @@ sub-templates) are in the character set I<charset>.
 I<charset> can be any character set that can be used with the module L<Encode>. 
 
 
-=head2 Global Variables
-
-If you want to use an option throughout your entire program and don't want to
-have to pass it to the constructor each time, you can set them globally. They
-will then act as defaults unless you override them in the constructor.
-
-  $Petal::BASE_DIR           (use base_dir option)
-  $Petal::INPUT              (use input option)
-  $Petal::OUTPUT             (use output option)
-  $Petal::TAINT              (use taint option)
-  $Petal::ERROR_ON_UNDEF_VAR (use error_on_undef_var option)
-  $Petal::DISK_CACHE         (use disk_cache option)
-  $Petal::MEMORY_CACHE       (use memory_cache option)
-  $Petal::MAX_INCLUDES       (use max_includes option)
-  $Petal::LANGUAGE           (use default_language option)
-  $Petal::DEBUG_DUMP         (use debug_dump option)
-    # $Petal::ENCODE_CHARSET     (use encode_charset option) -- _DEPRECATED_
-  $Petal::DECODE_CHARSET     (use decode_charset option)
-
-
 =head1 TAL SYNTAX
 
 This functionality is directly and shamelessly stolen from the excellent TAL
-specification: L<http://www.zope.org/Wikis/DevSite/Projects/ZPT/TAL>.
+specification: L<http://wiki.zope.org/ZPT/TAL>.
 
 
 =head2 define
@@ -1086,15 +1056,89 @@ Abstract
      blah blah blah
   </tag>
 
-Example:
-
-  <li tal:repeat="user system/user_list">$user/real_name</li>
-
 Why?
 
 Repeat statements are used to loop through a list of values,
 typically to display the resulting records of a database query.
 
+Example:
+
+  <li tal:repeat="user system/user_list">$user/real_name</li>
+
+A select list with one item selected:
+
+  <select>
+    <option
+      selected="selected"
+      tal:attributes="value self/selected_lang/value"
+      tal:content="self/selected_lang/label"
+    >English</option>
+    <option
+      value="i-klingon"
+      tal:repeat="lang self/unselected_langs;"
+      tal:attributes="value lang/value"
+      tal:content="lang/label"
+    >Klingon</option>
+  </select>
+
+A table with rows of alternating colours set via CSS:
+
+  <table>
+    <div
+      tal:omit-tag=""
+      tal:repeat="audience self/audiences"
+    >
+      <tr 
+        class="odd"
+        tal:condition="repeat/odd"
+      >
+        <td>
+          This a odd row, it comes before the even row.
+        </td>
+      </tr>
+      <tr 
+        class="even"
+        tal:condition="repeat/even"
+      >
+        <td> 
+          This a even row.
+        </td>
+      </tr>
+    </div>
+  </table>
+
+I<repeat> is a local temporary object that only exists within a
+petal:repeat loop.  It has a bunch of methods useful for selecting
+different positions in the loop:
+
+=head3 repeat/index
+
+I<index> returns the numeric position of this item within the loop, starts with
+one not zero.
+
+=head3 repeat/number
+
+I<number> is an alias for I<index>.
+
+=head3 repeat/even
+
+I<even> is true if the position is even (0, 2, 4 ...)
+
+=head3 repeat/odd
+
+I<odd> is true is the position is odd (1, 3, 5 ...)
+
+=head3 repeat/start
+
+I<start> is true if this is the first item.
+
+=head3 repeat/end
+
+I<end> is true if this is the last item.
+
+=head3 repeat/inner
+
+I<inner> is true if this is not the I<start> or I<end>.
 
 =head2 attributes
 
@@ -1189,6 +1233,10 @@ If you want to ALWAYS remove a tag, you can use C<omit-tag="string:1">
 
 =head2 on-error
 
+Warning: this is currently only partially implemented.  C<on-error> may be used
+in Petal templates, but the expression isn't evaluated - Petal simply prints
+the expression as a string.
+
 Abstract
 
   <tag on-error="EXPRESSION">...</tag>
@@ -1232,20 +1280,6 @@ statements will be executed in the following order:
         content
 
 
-=head2 aliases
-
-On top of all that, for people who are lazy at typing the following
-aliases are provided (although I would recommend sticking to the
-defaults):
-
-  * tal:define     - tal:def, tal:set
-  * tal:condition  - tal:if
-  * tal:repeat     - tal:for, tal:loop, tal:foreach
-  * tal:attributes - tal:att, tal:attr, tal:atts
-  * tal:content    - tal:inner
-  * tal:replace    - tal:outer
-
-
 TRAP:
 
 Don't forget that the default prefix is C<petal:> NOT C<tal:>, until
@@ -1254,79 +1288,21 @@ you set the petal namespace in your HTML or XML document as follows:
     <html xmlns:tal="http://purl.org/petal/1.0/">
 
 
-=head1 INCLUDES
+=head1 METAL MACROS
 
-Let's say that your base directory is C</templates>,
-and you're editing C</templates/hello/index.html>.
-
-From there you want to include C</templates/includes/header.html>
-
-
-=head2 general syntax
-
-You can use a subset of the XInclude syntax as follows:
-
-  <body xmlns:xi="http://www.w3.org/2001/XInclude">
-    <xi:include href="/includes/header.html" />
-  </body>
-
-
-For backwards compatibility reasons, you can omit the first slash, i.e.
-
-  <xi:include href="includes/header.html" />
-
-
-=head2 relative paths
-
-If you'd rather use a path which is relative to the template itself rather
-than the base directory, you can do it but the path MUST start with a dot,
-i.e.
-
-  <xi:include href="../includes/header.html" />
-
-  <xi:include href="./subdirectory/foo.xml" />
-
-etc.
-
-
-=head2 limitations
-
-The C<href> parameter does not support URIs, no other tag than C<xi:include> is
-supported, and no other directive than the C<href> parameter is supported at
-the moment.
-
-Also note that contrarily to the XInclude specification Petal DOES allow
-recursive includes up to C<$Petal::MAX_INCLUDES>. This behavior is very useful
-when templating structures which fit well recursive processing such as trees,
-nested lists, etc.
-
-You can ONLY use the following Petal directives with Xinclude tags:
-
-  * on-error
-  * define
-  * condition
-  * repeat
-
-C<replace>, C<content>, C<omit-tag> and C<attributes> are NOT supported in
-conjunction with XIncludes.
-
-
-=head1 MORE INCLUDES: METAL
-
-Petal now supports a subset of the METAL specification, which is a very WYSIWYG
-compatible way of doing includes. The current implementation supports only two
-metal statements: define-macro and use-macro.
+Petal supports an implementation of the METAL specification, which is a very
+WYSIWYG compatible way of doing template includes.
 
 
 =head2 define-macro
 
 In order to define a macro inside a file (i.e. a fragment to be included), you
-use the metal:use-macro directive. For example:
+use the metal:define-macro directive. For example:
 
   File foo.xml
   ============
 
-  <html>
+  <html xmlns:metal="http://xml.zope.org/namespaces/metal">
     <body>
       <p metal:define-macro="footer">
         (c) Me (r)(tm) (pouet pouet)
@@ -1337,13 +1313,13 @@ use the metal:use-macro directive. For example:
 
 =head2 use-macro
 
-In order to use a previously defined macro, you use the metal:define-macro directive.
+In order to use a previously defined macro, you use the metal:use-macro directive.
 For example:
 
   File bar.xml
   ============
 
-  <html>
+  <html xmlns:metal="http://xml.zope.org/namespaces/metal">
     <body>
       ... plenty of content ...
 
@@ -1354,6 +1330,54 @@ For example:
   </html>
 
 
+=head2 define-slot
+
+In any given macro you can define slots, which are bits of macros that can be
+overridden by something else using the fill-macro directive. To re-use the
+example above, imagine that we want to be able to optionally override the
+(pouet pouet) bit with something else:
+
+
+  File foo.xml
+  ============
+
+  <html xmlns:metal="http://xml.zope.org/namespaces/metal">
+    <body>
+      <p metal:define-macro="footer">
+        (c) Me (r)(tm) <span metal:define-slot="pouet">(pouet pouet)</span>
+      </p>
+    </body>
+  </html>
+
+
+=head2 fill-slot
+
+Your including file can override any slot using the fill-slot instruction, i.e.
+
+  File bar.xml
+  ============
+
+  <html xmlns:metal="http://xml.zope.org/namespaces/metal">
+    <body>
+      ... plenty of content ...
+
+      <p metal:use-macro="foo.xml#footer">
+        Page Footer. <span metal:fill-slot="pouet" petal:omit-tag="">(bar baz)</span>
+      </p>
+    </body>
+  </html>
+
+This would result in the macro 'foo.xml#footer' to produce:
+
+  <html>
+    <body>
+      <p>
+        (c) Me (r)(tm) (bar baz)
+      </p>
+    </body>
+  </html>
+
+
 =head2 self includes
 
 In Zope, METAL macros are expanded first, and then the TAL instructions are processed.
@@ -1363,7 +1387,8 @@ which allows for recursive macros.
 This example templates a sitemap, which on a hierarchically organized site would
 be recursive by nature:
 
-  <html>
+  <html xmlns:metal="http://xml.zope.org/namespaces/metal"
+        xmlns:petal="http://purl.org/petal/1.0/">
     <body>
       <p>Sitemap:</p>
 
@@ -1531,7 +1556,7 @@ I'm pretty sure you can work this one out by yourself :-)
 =head2 set:variable_name EXPRESSION
 
 Sets the value returned by the evaluation of EXPRESSION in
-C<$hash->{variable_name}>. For instance:
+C<$hash-E<gt>{variable_name}>. For instance:
 
 Perl expression:
 
@@ -1559,142 +1584,6 @@ expressions which invoke methods with parameters, i.e.
   string:The current CGI 'action' param is: ${cgi/param --action}
 
 
-=head1 UGLY SYNTAX
-
-For certain things which are not doable using TAL you can use what
-I call the UGLY SYNTAX. The UGLY SYNTAX is UGLY, but it can be handy
-in some cases.
-
-For example consider that you have a list of strings:
-
-    $my_var = [ 'Foo', 'Bar', 'Baz' ];
-    $template->process (my_var => $my_var, buz => $buz);
-
-
-And you want to display:
-
-  <title>Hello : Foo : Bar : Baz</title>
-
-Which is not doable with TAL without making the XHTML invalid.
-With the UGLY SYNTAX you can do:
-
-    <title>Hello<?for name="string my_var"?> : <?var name="string"?><?end?></title>
-
-Of course you can freely mix the UGLY SYNTAX with other Petal
-syntaxes. So:
-
-    <title><?for name="string my_var"?> $string <?end?></title>
-
-Mind you, if you've managed to read the doc this far I must confess
-that writing:
-
-    <h1>$string</h1>
-
-instead of:
-
-    <h1 tal:replace="string">Dummy</h1>
-
-is UGLY too. I would recommend to stick with TAL wherever you can.
-But let's not disgress too much.
-
-
-=head2 variables
-
-Abstract
-
-  <?var name="EXPRESSION"?>
-
-Example
-
-  <title><?var name="document/title"?></title>
-
-Why?
-
-Because if you don't have things which are replaced by real values in your
-template, it's probably a static page, not a template... :) 
-
-
-=head2 if / else constructs
-
-Usual stuff:
-
-  <?if name="user/is_birthay"?>
-    Happy Birthday, $user/real_name!
-  <?else?>
-    What?! It's not your birthday?
-    A very merry unbirthday to you! 
-  <?end?>
-
-You can use C<condition> instead of C<if>, and indeed you can use modifiers:
-
-  <?condition name="false:user/is_birthay"?>
-    What?! It's not your birthday?
-    A very merry unbirthday to you! 
-  <?else?>
-    Happy Birthday, $user/real_name!
-  <?end?>
-
-Not much else to say!
-
-
-=head2 loops
-
-Use either C<for>, C<foreach>, C<loop> or C<repeat>. They're all the same
-thing, which one you use is a matter of taste. Again no surprise:
-
-  <h1>Listing of user logins</h1>
-  <ul>
-    <?repeat name="user system/list_users"?>
-      <li><?var name="user/login"?> :
-          <?var name="user/real_name"?></li>
-    <?end?>
-  </ul>
-  
-
-Variables are scoped inside loops so you don't risk to erase an existing
-C<user> variable which would be outside the loop. The template engine also
-provides the following variables for you inside the loop:
-
-  <?repeat name="foo bar"?>
-    <?var name="repeat/index"?>  - iteration number, starting at 0
-    <?var name="repeat/number"?> - iteration number, starting at 1
-    <?var name="repeat/start"?>  - is it the first iteration?
-    <?var name="repeat/end"?>    - is it the last iteration?
-    <?var name="repeat/inner"?>  - is it not the first and not the last iteration?
-    <?var name="repeat/even"?>   - is the count even?
-    <?var name="repeat/odd"?>    - is the count odd?
-  <?end?>
-
-Again these variables are scoped, you can safely nest loops, ifs etc...  as
-much as you like and everything should be fine.
-
-
-=head2 includes
-
-The XInclude syntax should be preferred over this...
-
-  <?include file="include.xml"?>
-
-It will include the file 'include.xml', using the current C<@Petal::BASE_DIR>
-directory list.
-
-If you want use XML::Parser to include files, you should make sure that
-the included files are valid XML themselves... FYI XML::Parser chokes on
-this:
-
-    <p>foo</p>
-    <p>bar</p>
-
-But this works:
-
-    <div>
-      <p>foo</p>
-      <p>bar</p>
-    </div>
-
-(Having only one top element is part of the XML spec).
-
-
 =head1 ADVANCED PETAL
 
 
@@ -1837,13 +1726,18 @@ The cycle of a Petal template is the following:
     6. Petal turns the perl code into a subroutine
     7. Petal::Cache::Memory caches the subroutine in memory
     8. Petal executes the subroutine
+    9. (optional) Petal internationalizes the resulting output.
 
-If you are under a persistent environement a la mod_perl, subsequent calls to
+If you are under a persistent environment a la mod_perl, subsequent calls to
 the same template will be reduced to step 8 until the source template changes.
 
 Otherwise, subsequent calls will resume at step 6, until the source template
 changes.
 
+If you are using the mod_perl prefork MPM, you can precompile Petal templates
+into Apache's shared memory at startup by using the cache_only option.  This
+will allow you to run through steps 1-7 without passing any data to Petal.
+
 
 =head1 DECRYPTING WARNINGS AND ERRORS
 
@@ -1882,7 +1776,7 @@ try to use the $Petal::INPUT = 'XHTML' option.
 =head2 other errors
 
 Either I've forgot to document it, or it's a bug. Send an email to the Petal
-mailing list or at L<mailto://jhiver@mkdoc.com>.
+mailing list.
 
 
 =head1 EXPORTS
@@ -1890,22 +1784,11 @@ mailing list or at L<mailto://jhiver@mkdoc.com>.
 None.
 
 
-=head1 KNOWN BUGS
-
-The XML::Parser wrapper only cannot expand entities C<&lt;>, C<&gt;>, C<&amp;>
-and C<&quot;>. Besides, I can't get it to NOT expand entities in 'Stream' mode.
-
-HTML::TreeBuilder expands all entities, hence &nbsp;s are lost / converted to
-whitespaces.
-
-XML::Parser is deprecated and should be replaced by SAX handlers at some point.
-
-
 =head1 AUTHOR
 
-Copyright 2003 - MKDoc Holdings Ltd.
+Copyright 2003 - MKDoc Ltd.
 
-Authors: Jean-Michel Hiver <jhiver@mkdoc.com>, 
+Authors: Jean-Michel Hiver, 
          Fergal Daly <fergal@esatclear.ie>,
 	 and others.
 
@@ -1929,9 +1812,7 @@ Mailing list archives:
 
 Have a peek at the TAL / TALES / METAL specs:
 
-  http://www.zope.org/Wikis/DevSite/Projects/ZPT/TAL
-  http://www.zope.org/Wikis/DevSite/Projects/ZPT/TALES
-  http://www.zope.org/Wikis/DevSite/Projects/ZPT/METAL
+  http://wiki.zope.org/ZPT/TAL
+  http://wiki.zope.org/ZPT/TALES
+  http://wiki.zope.org/ZPT/METAL
 
-
-Any extra questions? jhiver@mkdoc.com.
